package org.brynhild.graduation.builder.executor

import jakarta.annotation.Resource
import org.brynhild.graduation.builder.event.BuildEventResultSender
import org.brynhild.graduation.builder.transfer.BuildDetail
import org.brynhild.graduation.builder.transfer.TaskInfo
import org.brynhild.graduation.common.constant.TriggerConstant
import org.brynhild.graduation.transfer.builder.BuildTaskResult
import org.brynhild.graduation.transfer.trigger.BuildTask
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component
import org.springframework.web.client.RestTemplate

@Component
class BuildTaskExecutor {
    private BuildEventResultSender sender
    private RestTemplate restTemplate
    private final String executorPath = "http://192.168.110.3:8888/job/HelloWorld"
    private final String token = "fetchCodeOnly"
    private final String fetchCodeOnlyHook = "${executorPath}/build?token=${token}"
    private final String fetchBuildTaskList = "${executorPath}/api/json"
    private volatile boolean busy = false

    synchronized void execute(BuildTask task) {
        def begin = System.currentTimeMillis()
        def modules = task.modules
        def result = new BuildTaskResult()
        result.with {
            taskId = task.taskId
            flag = false
        }
        if (busy) {
            sender.sendResult(result)
            println "已有构建任务，丢弃新任务"
            return
        }
        busy = true

        try {
            if (createFetchTask() < 0) {
                return
            }
            int status
            do {
                status = fetchCodeFinish()
                if (status == -1) {
                    return
                }
                if (status == 2) {
                    return
                }
            } while (status == 0)
            buildProject()
            for (module in modules) {
                if (TriggerConstant.dockerModuleSet.contains(module)) {
                    dockerBuildAndPush(module)
                }
            }
            result.flag = true
        } catch (Exception e) {
            e.printStackTrace()
        } finally {
            busy = false
            def end = System.currentTimeMillis()
            println "构建时间：${end - begin}ms"
            sender.sendResult(result)
        }
    }

    int createFetchTask() {
        def taskInfo = restTemplate.getForObject(fetchBuildTaskList, TaskInfo.class)
        if (!taskInfo.buildable) {
            println "不可构建"
            return -1
        }

        def entity = restTemplate.getForEntity(fetchCodeOnlyHook, Object.class)
        if (entity.statusCode.value() != 201) {
            println "构建状态异常"
            return -1
        }
        return 1
    }

    int fetchCodeFinish() {
        def taskInfo = restTemplate.getForObject(fetchBuildTaskList, TaskInfo.class)
        if (taskInfo == null) {
            println "无法获取taskInfo"
            return -1
        }
        if (taskInfo.builds == null || taskInfo.builds.isEmpty()) {
            println "构建队列为空"
            return -1
        }
        def build = taskInfo.builds.get(0)
        def number = build.number

        def detailHook = "${executorPath}/${number}/api/json"
        Thread.sleep(1000)
        def detail = restTemplate.getForObject(detailHook, BuildDetail.class)
        if (detail == null) {
            println "无法获取最新的构建信息"
            return -1
        }

        if (detail.building == true) {
            println "构建中"
            return 0
        } else {
            return 1
        }
    }

    static void buildProject() {
        String[] cmdArray = new String[]{"/usr/local/maven/bin/mvn", "clean", "package", "install"}
        try {
            Process process = Runtime.getRuntime().exec(cmdArray, null, new File("/usr/local/jenkins/workspace/HelloWorld"))
            InputStream is = process.getInputStream()
            InputStreamReader isr = new InputStreamReader(is, "GBK")
            BufferedReader br = new BufferedReader(isr)
            String content = br.readLine()
            while (content != null) {
                System.out.println(content)
                content = br.readLine()
            }
        } catch (Exception e) {
            e.printStackTrace()
        }
    }

    static void dockerBuildAndPush(String moduleName) {
        String[] cmdArray = new String[]{"/usr/local/maven/bin/mvn", "docker:build", "-DpushImage"}
        try {

            Process process = Runtime.getRuntime().exec(cmdArray, null, new File("/usr/local/jenkins/workspace/HelloWorld/${moduleName}"))
            InputStream is = process.getInputStream()
            InputStreamReader isr = new InputStreamReader(is, "GBK")
            BufferedReader br = new BufferedReader(isr)
            String content = br.readLine()
            while (content != null) {
                System.out.println(content)
                content = br.readLine()
            }
        } catch (Exception e) {
            e.printStackTrace()
        }
    }


    @Autowired
    @Resource
    void setSender(BuildEventResultSender sender) {
        this.sender = sender
    }

    @Autowired
    @Resource
    void setRestTemplate(RestTemplate restTemplate) {
        this.restTemplate = restTemplate
    }
}
