package org.brynhild.graduation.device.event;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.utils.StringUtil;
import org.brynhild.graduation.device.config.MQTTConfig;
import org.brynhild.graduation.device.event.callback.MqttDeviceCallback;
import org.brynhild.graduation.device.util.JwtValidator;
import org.brynhild.graduation.device.util.RedisUtil;
import org.brynhild.graduation.persistence.user.entity.User;
import org.brynhild.graduation.persistence.user.repository.UserRepository;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

@Component
@RequiredArgsConstructor
public class MQTTNotifyService {
    private final MQTTConfig config;
    private final RedisUtil redisUtil;
    private final JwtValidator jwtValidator;
    private final UserRepository userRepository;
    private Map<Long, MqttAccountInstance> accountInstanceMap = new HashMap<>();
    private MqttClient client;

    @PostConstruct
    public void initMainHandler() {
        try {
            MemoryPersistence persistence = new MemoryPersistence();
            System.out.println("注册新的Client:" + config.getClientId() + "-notifyService");
            client = new MqttClient(config.getBroker(), config.getClientId() + "-notifyService", persistence);
            MqttConnectOptions options = new MqttConnectOptions();
            options.setUserName(config.getUsername());
            options.setPassword(config.getPassword().toCharArray());
            options.setCleanSession(true);
            client.setCallback((MqttDeviceCallback) (topic, message) -> {
                System.out.println("Topic:" + topic);
                String content = new String(message.getPayload());
                System.out.println("Message:" + content);
                final Long userId = getUserId(content);
                final Optional<User> userOptional = userRepository.findById(userId);
                String returnMessage = "";
                String retTopic = "";
                if (userOptional.isEmpty()) {
                    returnMessage = "用户信息不存在";
                } else {
                    final User user = userOptional.get();
                    retTopic = user.getUsername() + "-notifyService";
                    registerNewHandler(userId, retTopic, retTopic);
                    returnMessage = "OK";
                }
                MqttMessage mqttMessage = new MqttMessage(returnMessage.getBytes());
                mqttMessage.setQos(config.getQos());
                try {
                    if (!StringUtil.isEmpty(retTopic)) {
                        System.out.println("SEND:" + returnMessage);
                        client.publish(retTopic, mqttMessage);
                        System.out.println("SEND:" + returnMessage + " FINISHED");
                    }
                } catch (MqttException me) {
                    System.out.println("reason " + me.getReasonCode());
                    System.out.println("msg " + me.getMessage());
                    System.out.println("loc " + me.getLocalizedMessage());
                    System.out.println("cause " + me.getCause());
                    System.out.println("excep " + me);
                    me.printStackTrace();
                }
            });

            client.connect(options);
            client.subscribe("android");

        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("MQTT Notify Service Register Finished!");
    }

    private Long getUserId(String token) {
        return Long.parseLong(jwtValidator.getId(token));
    }

    private void registerNewHandler(Long userId, String topic, String retTopic) {
        System.out.println("User Register:" + userId);
        try {
            MemoryPersistence persistence = new MemoryPersistence();
            System.out.println("注册新的Client:" + topic);
            MqttClient client = new MqttClient(config.getBroker(), topic, persistence);
            MqttConnectOptions options = new MqttConnectOptions();
            options.setUserName(config.getUsername());
            options.setPassword(config.getPassword().toCharArray());
            options.setCleanSession(true);

            client.setCallback((MqttDeviceCallback) (topic1, message) -> {

            });
            client.connect(options);
            System.out.println("New Notify Handler listen on:" + topic);
            client.subscribe(topic);

            MqttAccountInstance instance = new MqttAccountInstance(userId, client, persistence, retTopic);

            accountInstanceMap.put(userId, instance);
        } catch (MqttException me) {
            System.out.println("reason " + me.getReasonCode());
            System.out.println("msg " + me.getMessage());
            System.out.println("loc " + me.getLocalizedMessage());
            System.out.println("cause " + me.getCause());
            System.out.println("excep " + me);
            me.printStackTrace();
        }
    }

    public boolean notifyClient(Long userId, String message) {
        final MqttAccountInstance instance = accountInstanceMap.get(userId);
        if (instance != null) {
            MqttMessage mqttMessage = new MqttMessage(message.getBytes());
            mqttMessage.setQos(config.getQos());
            return instance.sendMessage(mqttMessage);
        }
        return false;
    }

    @Data
    @AllArgsConstructor
    static class MqttAccountInstance {
        private final Long userId;
        private final MqttClient client;
        private final MemoryPersistence persistence;
        private final String retTopic;

        public boolean sendMessage(MqttMessage message) {
            try {
                System.out.println(retTopic + "==== START ====>" + message);
                client.publish(retTopic, message);
                System.out.println(retTopic + "==== FINISH ====>");
                return true;
            } catch (MqttException me) {
                System.out.println("reason " + me.getReasonCode());
                System.out.println("msg " + me.getMessage());
                System.out.println("loc " + me.getLocalizedMessage());
                System.out.println("cause " + me.getCause());
                System.out.println("excep " + me);
                me.printStackTrace();
                return false;
            }
        }
    }
}
