package org.brynhild.graduation.device.event;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.constant.MqttConstant;
import org.brynhild.graduation.common.utils.TimeUtil;
import org.brynhild.graduation.device.config.MQTTConfig;
import org.brynhild.graduation.device.event.callback.MqttDeviceCallback;
import org.brynhild.graduation.device.util.MqttUtil;
import org.brynhild.graduation.device.util.RedisUtil;
import org.brynhild.graduation.persistence.device.entity.Area;
import org.brynhild.graduation.persistence.device.entity.Device;
import org.brynhild.graduation.persistence.device.entity.FingerPrintData;
import org.brynhild.graduation.persistence.device.entity.SignInRecord;
import org.brynhild.graduation.persistence.device.repository.AreaRepository;
import org.brynhild.graduation.persistence.device.repository.DeviceRepository;
import org.brynhild.graduation.persistence.device.repository.FingerPrintDataRepository;
import org.brynhild.graduation.persistence.device.repository.SignInRecordRepository;
import org.brynhild.graduation.persistence.user.entity.User;
import org.brynhild.graduation.persistence.user.repository.UserRepository;
import org.brynhild.graduation.transfer.device.bo.NetworkConfig;
import org.brynhild.graduation.transfer.device.bo.SignInRecordBinary;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

@Component
@RequiredArgsConstructor
public class MQTTHandler {

    private static Integer qos;
    private final MQTTConfig config;
    private final RedisUtil redisUtil;
    private final DeviceRepository deviceRepository;

    private final UserRepository userRepository;

    private final AreaRepository areaRepository;

    private final FingerPrintDataRepository fingerPrintDataRepository;

    private final SignInRecordRepository signInRecordRepository;

    private final MQTTNotifyService notifyService;
    private MqttClient client;
    @Getter
    private Map<String, MqttDeviceInstance> deviceInstanceMap = new HashMap<>();
    private Map<Integer, MqttDeviceCallback> defaultCallbackMap = new HashMap<>();

    public boolean sendMessageToDevice(String macAddress, String message) {
        return sendMessageToDevice(macAddress, message.getBytes());
    }

    public boolean sendMessageToDevice(String macAddress, byte[] message) {
        System.out.println("[INFO]Message Length:" + message.length);
        final MqttDeviceInstance instance = deviceInstanceMap.get(macAddress);
        if (instance == null) {
            return false;
        }
        MqttMessage mqttMessage = new MqttMessage(message);
        mqttMessage.setQos(config.getQos());
//        try{
//            client.publish(MqttUtil.getServerPublish(macAddress), mqttMessage);
//            return true;
//        }catch (Exception e){
//            e.printStackTrace();
//            return false;
//        }
        return instance.sendMessage(mqttMessage);
    }

    @PostConstruct
    private void registerMainHandler() {
        try {
            initDefaultCallbackMap();
            MemoryPersistence persistence = new MemoryPersistence();
            System.out.println("注册新的Client:" + config.getClientId());
            client = new MqttClient(config.getBroker(), config.getClientId(), persistence);
            MqttConnectOptions options = new MqttConnectOptions();
            qos = config.getQos();
            options.setUserName(config.getUsername());
            options.setPassword(config.getPassword().toCharArray());
            options.setCleanSession(true);
            client.setCallback((MqttDeviceCallback) (topic, message) -> {
                try {
                    System.out.println("Topic:" + topic);
                    String content = new String(message.getPayload());
                    System.out.println("Message:" + content);
                    final int deviceType = MqttUtil.getDeviceType(content);
                    String deviceMac = MqttUtil.getDeviceMac(content);

                    String returnMessage = "";
                    if (deviceType == MqttConstant.FINAL) {
                        Device device = deviceRepository.findDeviceByMac(deviceMac);
                        if (device == null) {
                            String verifyCode = redisUtil.getVerifyCode(MqttConstant.VERIFY_CODE_PREFIX, deviceMac);
                            returnMessage = "verifyCode:" + verifyCode;
                        } else {
                            device.setLastActive(TimeUtil.getCurrentTimeString());
                            deviceRepository.save(device);
                            returnMessage = "permitRegister:" + device.getMac();
                        }
                    }
                    MqttMessage mqttMessage = new MqttMessage(returnMessage.getBytes());
                    mqttMessage.setQos(config.getQos());
                    registerNewHandler(deviceMac, defaultCallbackMap.get(deviceType));
                    try {
                        System.out.println("SEND:" + returnMessage);
                        client.publish(MqttUtil.getServerPublish(deviceMac), mqttMessage);
                        System.out.println("SEND:" + returnMessage + " FINISHED");
                    } catch (MqttException me) {
                        System.out.println("reason " + me.getReasonCode());
                        System.out.println("msg " + me.getMessage());
                        System.out.println("loc " + me.getLocalizedMessage());
                        System.out.println("cause " + me.getCause());
                        System.out.println("excep " + me);
                        me.printStackTrace();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
            client.connect(options);
            client.subscribe(config.getListen());
        } catch (Exception e) {
            e.printStackTrace();
        }


        System.out.println("MQTT Main Handler Register Finished!");
    }

    private void registerNewHandler(String macAddress, MqttDeviceCallback callback) {
        System.out.println("Device Register:" + macAddress);
        try {
            MemoryPersistence persistence = new MemoryPersistence();
            System.out.println("注册新的Client:" + config.getClientId() + "-handler-" + macAddress);
            MqttClient client = new MqttClient(config.getBroker(), config.getClientId() + "-handler-" + macAddress, persistence);
            MqttConnectOptions options = new MqttConnectOptions();
            options.setUserName(config.getUsername());
            options.setPassword(config.getPassword().toCharArray());
            options.setCleanSession(true);

            client.setCallback(callback);
            client.connect(options);
            System.out.println("New handler listen on:" + MqttUtil.getServerListen(macAddress));
            System.out.println("New handler connect state:" + client.isConnected());

            client.subscribe(MqttUtil.getServerListen(macAddress));

            MqttDeviceInstance instance = new MqttDeviceInstance(macAddress, client, persistence);
            deviceInstanceMap.put(macAddress, instance);

        } catch (MqttException me) {
            System.out.println("reason " + me.getReasonCode());
            System.out.println("msg " + me.getMessage());
            System.out.println("loc " + me.getLocalizedMessage());
            System.out.println("cause " + me.getCause());
            System.out.println("excep " + me);
            me.printStackTrace();
        }
    }

    private void initDefaultCallbackMap() {
        defaultCallbackMap.put(MqttConstant.FINAL, (topic, message) -> {
            final byte[] payload = message.getPayload();
            final String content = new String(payload);
            final String macAddress = MqttUtil.getMacAddressFromTopic(topic);
            System.out.println("content:" + content + ", mac:" + macAddress);
            final Device device = deviceRepository.findDeviceByMac(macAddress);

            if (device == null) {
                return;
            }
            device.setLastActive(TimeUtil.getCurrentTimeString());
            deviceRepository.save(device);
            try {
                if (payload[0] == MqttConstant.UPLOAD_FINGER_PRINT) {
                    byte[] copy = Arrays.copyOfRange(payload, 1, payload.length);
                    System.out.println(copy.length);
                    if (copy.length == 536) {
                        System.out.println("UPLOAD FINGER PRINT ACTION");
                        FingerPrintData data = new FingerPrintData(copy);
                        fingerPrintDataRepository.save(data);
                    }
                }
                if (payload[0] == MqttConstant.UPLOAD_SIGNIN_RECORD) {
                    byte[] copy = Arrays.copyOfRange(payload, 1, payload.length);
                    System.out.println(copy.length);
                    if (copy.length == 24) {
                        System.out.println("UPLOAD SIGN IN RECORD ACTION");
                        SignInRecordBinary recordBinary = new SignInRecordBinary(copy);
                        SignInRecord record = new SignInRecord();
                        record.setUserId(recordBinary.getUid().longValue());
                        record.setAreaId(device.getArea());
                        record.setDeviceId(device.getId());
                        Optional<User> userOptional = userRepository.findById(record.getUserId());
                        Optional<Area> areaOptional = areaRepository.findById(record.getAreaId());
                        userOptional.ifPresent(user -> record.setUserName(user.getName()));
                        areaOptional.ifPresent(area -> record.setAreaName(area.getName()));
                        record.setDeviceName(device.getName());
                        signInRecordRepository.save(record);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (content.equals("fetchConfiguration")) {
                NetworkConfig networkConfig = new NetworkConfig(device);
                try {
                    client.publish(MqttUtil.getServerPublish(macAddress), networkConfig.convertToBytes(), config.getQos(), false);
                } catch (MqttException me) {
                    System.out.println("reason " + me.getReasonCode());
                    System.out.println("msg " + me.getMessage());
                    System.out.println("loc " + me.getLocalizedMessage());
                    System.out.println("cause " + me.getCause());
                    System.out.println("excep " + me);
                    me.printStackTrace();
                }
            }
            if (content.equals("restartDependentDevices")) {
                //IGNORE
            }


            redisUtil.setDeviceReportData(macAddress, content);
            redisUtil.refreshDeviceHeartBeat(macAddress);
        });
    }

    @AllArgsConstructor
    @Data
    static class MqttDeviceInstance {
        private final String macAddress;
        private final MqttClient client;
        private final MemoryPersistence persistence;

        public boolean sendMessage(MqttMessage message) {
            try {
                System.out.println("Send Message To:" + MqttUtil.getServerPublish(macAddress));
                client.publish(MqttUtil.getServerPublish(macAddress), message.getPayload(), message.getQos(), false);
                System.out.println("Send Message Finished");
                return true;
            } catch (MqttException me) {
                System.out.println("reason " + me.getReasonCode());
                System.out.println("msg " + me.getMessage());
                System.out.println("loc " + me.getLocalizedMessage());
                System.out.println("cause " + me.getCause());
                System.out.println("excep " + me);
                me.printStackTrace();
                return false;
            }
        }
    }


}
