package org.brynhild.graduation.device.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.constant.DeviceConstant
import org.brynhild.graduation.common.constant.MqttConstant
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.common.utils.StringUtil
import org.brynhild.graduation.device.event.LogSender
import org.brynhild.graduation.device.event.MQTTHandler
import org.brynhild.graduation.device.factory.OperationLogFactory
import org.brynhild.graduation.device.factory.SpecificationFactory
import org.brynhild.graduation.device.service.IAdminService
import org.brynhild.graduation.persistence.device.entity.*
import org.brynhild.graduation.persistence.device.repository.*
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.user.entity.User
import org.brynhild.graduation.persistence.user.repository.ClassRepository
import org.brynhild.graduation.persistence.user.repository.UserRepository
import org.brynhild.graduation.transfer.device.vo.*
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminServiceImpl implements IAdminService {
    private SpecificationFactory specificationFactory
    private OperationLogFactory operationLogFactory
    private RedisTemplate<String, Object> redisTemplate
    private JwtUtil jwtUtil
    private LogSender logSender
    private MQTTHandler mqttHandler

    private DeviceRepository deviceRepository
    private DeviceRelationshipRepository relationshipRepository
    private AreaRepository areaRepository
    private CredentialRepository credentialRepository
    private SignInRecordRepository signInRecordRepository
    private UserRepository userRepository
    private FingerPrintDataRepository fingerPrintDataRepository
    private ClassRepository classRepository

    @Override
    Result addDevice(AdminAddDevice bindDevice, String token) {
        def mac = redisTemplate.opsForValue().get(MqttConstant.VERIFY_CODE_PREFIX + bindDevice.code) as String
        if (mac == null) {
            return new Result(false, "查询失败，验证码不存在")
        }
        if (!bindDevice.mac.equals(mac)) {
            return new Result(false, "信息不匹配")
        }
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def device = new Device()

        if (bindDevice.getType() == DeviceConstant.PRIMARY_GATEWAY) {
            if (bindDevice.getMaster() != null) {
                return new Result(false, "一级网关不允许有上级设备")
            }
        } else {
            def optional = deviceRepository.findById(bindDevice.getMaster())
            if (optional.isEmpty()) {
                return new Result(false, "上级设备不存在")
            }
            def master = optional.get()
            if (master.getType() <= bindDevice.getType()) {
                return new Result(false, "无法绑定低级或同级设备")
            }
        }

        if (bindDevice.getType() != DeviceConstant.TERMINAL) {
            if (StringUtil.isEmpty(bindDevice.apSsid) ||
                    StringUtil.isEmpty(bindDevice.apPassword) ||
                    StringUtil.isEmpty(bindDevice.staSsid) ||
                    StringUtil.isEmpty(bindDevice.staPassword)) {
                return new Result(false, "网络配置信息不完整")
            }
        } else {
            if (StringUtil.isEmpty(bindDevice.staSsid) ||
                    StringUtil.isEmpty(bindDevice.staPassword)) {
                return new Result(false, "网络配置信息不完整")
            }
        }

        if (deviceRepository.findDeviceByMac(bindDevice.mac)) {
            return new Result(false, "该设备已被绑定过");
        }


        def log = operationLogFactory.generateLogWithBindDevice(bindDevice, device)
        device.owner = id
        deviceRepository.save(device)

        log.with {
            operatorName = name
            operatorId = id
            targetName = device.name
            targetId = device.id
        }
        logSender.sendOperationLog(log)
        return new Result(true, "绑定成功")
    }

    @Override
    Result enrollFingerPrint(EnrollFingerPrint info, String token) {
        def data = fingerPrintDataRepository.findFingerPrintDataByUid(info.userId.toInteger())
        if (data != null) {
            return new Result(false, "该用户已录入指纹信息")
        }

        String message = "enrollFingerPrint:" + info.userId
        def sendResult = mqttHandler.sendMessageToDevice(info.mac, message)
        if (sendResult) {
            return new Result(true, "已通知设备执行录入指纹操作")
        } else {
            return new Result(false, "出现异常，信息不可达")
        }
    }

    @Override
    Result findSignInRecord(QuerySignInRecordRequest request) {
        def specification = specificationFactory.getSignInRecordSpecification(request)
        println request
        def list = signInRecordRepository.findAll(specification)
        list=list.sort()
        return new Result(true, "查询成功", list)
    }

    @Override
    Result findCredential(QueryCredentialRequest request) {
        def specification = specificationFactory.getCredentialSpecification(request)
        return new Result(true, "查询成功", credentialRepository.findAll(specification))
    }

    @Override
    Result findArea(QueryAreaRequest request) {
        def specification = specificationFactory.getAreaSpecification(request)
        return new Result(true, "查询成功", areaRepository.findAll(specification))
    }

    @Override
    Result findDevice(QueryDeviceRequest request) {
        def specification = specificationFactory.getDeviceSpecification(request)
        return new Result(true, "查询成功", deviceRepository.findAll(specification))
    }

    @Override
    Result modifyDevice(AdminModifyDevice info, String token) {
        def deviceOptional = deviceRepository.findById(info.id)
        if (deviceOptional.isEmpty()) {
            return new Result(false, "目标不存在")
        }
        def device = deviceOptional.get()

        if (info.getType() == DeviceConstant.PRIMARY_GATEWAY) {
            if (info.getMaster() != null) {
                return new Result(false, "一级网关不允许有上级设备")
            }
        } else {
            def optional = deviceRepository.findById(info.getMaster())
            if (optional.isEmpty()) {
                return new Result(false, "上级设备不存在")
            }
            def master = optional.get()
            if (master.getType() <= device.getType()) {
                return new Result(false, "无法绑定低级或同级设备")
            }
        }

        def log = operationLogFactory.generateLogWithAdminModifyDevice(info, device)
        deviceRepository.save(device)

        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        log.with {
            operatorName = name
            operatorId = id
            targetId = info.id
            targetName = device.name
        }
        logSender.sendOperationLog(log)
        return new Result(true, "修改设备成功")

    }

    @Override
    Result modifyArea(AdminModifyArea info, String token) {
        def areaOptional = areaRepository.findById(info.id)
        if (areaOptional.isEmpty()) {
            return new Result(false, "目标不存在")
        }
        def area = areaOptional.get()
        def log = operationLogFactory.generateLogWithAdminModifyArea(info, area)
        areaRepository.save(area)

        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        log.with {
            operatorName = name
            operatorId = id
            targetId = info.id
            targetName = area.name
        }
        logSender.sendOperationLog(log)
        return new Result(true, "修改设备成功")
    }

    @Override
    Result addCredential(AdminAddCredential info, String token) {
        def userOptional = userRepository.findById(info.userId)
        def areaOptional = areaRepository.findById(info.areaId)
        def deviceOptional = deviceRepository.findById(info.deviceId)
        if (userOptional.isEmpty() || areaOptional.isEmpty() || deviceOptional.isEmpty()) {
            return new Result(false, "目标不存在")
        }
        def user = userOptional.get()
        def area = areaOptional.get()
        def device = deviceOptional.get()

        def credential = new Credential()
        def log = operationLogFactory.generateLogWithAdminAddCredential(info, credential)

        credential.with {
            userName = user.username
            areaName = area.name
            deviceName = device.name
            deviceMac = device.mac
        }

        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        credentialRepository.save(credential)
        log.with {
            operatorId = id
            operatorName = name
            targetId = credential.id
            targetName = "${user.name}的凭据"
        }
        logSender.sendOperationLog(log)

        return new Result(true, "查询成功")
    }

    @Override
    Result addArea(AdminAddArea info, String token) {
        def area = new Area()
        def log = operationLogFactory.generateLogWithAdminAddArea(info, area)
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        areaRepository.save(area)
        log.with {
            targetName = area.name
            targetId = area.id
            operatorId = id
            operatorName = name
        }
        logSender.sendOperationLog(log)
        return new Result(true, "添加成功")
    }

    @Override
    Result deleteDevice(AdminDeleteDevice info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> logs = new ArrayList<>()

        for (Long did : info.id) {
            def optional = deviceRepository.findById(did)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("无相关数据")
            }

            def device = optional.get()
            deviceRepository.logicDelete(did)
            def log = new OperationLog()
            log.with {
                targetName = device.name
                targetId = device.id
                operatorId = id
                operatorName = name
            }
            logs.add(log)
        }


        for (OperationLog log : logs) {
            logSender.sendOperationLog(log)
        }

        return new Result(true, "删除成功")
    }

    @Override
    Result deleteArea(AdminDeleteArea info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        List<OperationLog> logs = new ArrayList<>()

        for (Long aid : info.id) {
            def optional = areaRepository.findById(aid)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("无相关数据")
            }

            def area = optional.get()
            areaRepository.logicDelete(aid)
            def log = new OperationLog()
            log.with {
                targetName = area.name
                targetId = area.id
                operatorId = id
                operatorName = name
                it.description = "管理员删除了区域"
            }
            logs.add(log)
        }


        for (OperationLog log : logs) {
            logSender.sendOperationLog(log)
        }
        return new Result(true, "删除成功")
    }

    @Override
    Result deleteCredential(AdminDeleteCredential info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> logs = new ArrayList<>()

        for (Long cid : info.id) {
            def optional = credentialRepository.findById(cid)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("无相关数据")
            }
            def credential = optional.get()
            deviceRepository.logicDelete(cid)
            def log = new OperationLog()
            log.with {
                targetName = credential.userName
                targetId = credential.id
                operatorId = id
                operatorName = name
                description = "管理员删除了凭证信息"
            }
            logs.add(log)
        }

        for (OperationLog log : logs) {
            logSender.sendOperationLog(log)
        }
        return new Result(true, "删除成功")
    }

    @Override
    Result addRelationship(AdminAddRelationship info, String token) {
        def master = deviceRepository.findById(info.master)
        def slave = deviceRepository.findById(info.slave)
        if (master.isEmpty() || slave.isEmpty()) {
            return new Result(false, "相关设备不存在")
        }
        master = master.get()
        slave = slave.get()

        def relationship = relationshipRepository.findRelationshipBySlave(info.slave)
        if (relationship != null) {
            return new Result(false, "从设备已被指派")
        }

        if (master.type <= slave.type) {
            return new Result(false, "指派类型不合法")
        }

        relationship = new DeviceRelationship()
        relationship.masterType = master.type
        def log = operationLogFactory.generateLogWithAdminAddRelationship(info, relationship)

        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        relationshipRepository.save(relationship)
        log.with {
            targetName = slave.name
            targetId = slave.id
            operatorId = id
            operatorName = name
        }

        logSender.sendOperationLog(log)

        return null
    }

    @Override
    Result findRelationship(QueryRelationshipRequest info) {
        return new Result(true, "查询成功", relationshipRepository.findAll(specificationFactory.getDeviceRelationshipSpecification(info)))
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result deleteRelationship(AdminDeleteRelationship info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        List<OperationLog> logs = new ArrayList<>();

        for (Long rid : info.id) {
            def optional = relationshipRepository.findById(rid)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("无相关数据")
            }
            def relationship = optional.get()
            relationshipRepository.logicDelete(rid)

            def log = new OperationLog()
            log.with {
                targetName = "${relationship.id}"
                targetId = rid
                operatorId = id
                operatorName = name
                description = "管理员解除了设备绑定"
            }
            logs.add(log)
        }

        for (OperationLog log : logs) {
            logSender.sendOperationLog(log)
        }
        return new Result(true, "删除成功")
    }

    @Override
    Result getClassFingerPrintDataIdList(AdminGetClassFingerPrintDataIdList info, String token) {
        def optional = classRepository.findById(info.id)
        if (!optional.isPresent()) {

        }
        def users = userRepository.findUserByDepartmentId(info.id)
        StringBuilder builder = new StringBuilder()
        boolean first = true
        boolean pushable = true
        def ret = new ArrayList<Long>()
        for (User user : users) {
            def fingerPrintData = fingerPrintDataRepository.findFingerPrintDataByUid(user.id as Integer)
            if (fingerPrintData == null) {
                if (!first) {
                    builder.append("、")
                } else {
                    first = false
                }
                builder.append(user.name)
                pushable = false
            } else {
                ret.add(fingerPrintData.id)
            }
        }
        if (pushable) {
            return new Result(true, "查询成功", ret)
        } else {
            return new Result(false, "推送失败，以下用户未录入指纹：" + builder.toString())
        }
    }

    @Override
    Result clearFingerPrintDataAtDevice(AdminClearFingerPrintDataAtDevice info, String token) {
        println info.id
        def device = deviceRepository.findDeviceByMac(info.mac)
        if (device == null) {
            return new Result(false, "设备不存在")
        }
        def optional = classRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "班级状态异常")
        }
        def clazz = optional.get()
        def users = userRepository.findUserByDepartmentId(info.id)
        def result = mqttHandler.sendMessageToDevice(info.mac,
                "${MqttConstant.DELETE_ALL_FINGER_PRINT},count:${users.size()},classNo:${info.id}")
        if (result) {
            return new Result(true, "清除指纹数据已执行")
        }
        return new Result(false, "信息发送失败")
    }

    @Override
    Result pushFingerPrintDataToDevice(AdminPushFingerPrintDataToDevice info, String token) {
        def device = deviceRepository.findDeviceByMac(info.mac)
        if (device == null) {
            return new Result(false, "设备不存在")
        }
        def listOfData = fingerPrintDataRepository.findAllById(info.id)
        if (listOfData.size() != info.id.size()) {
            return new Result(false, "数据不一致")
        }

        for (FingerPrintData data : listOfData) {
            def bytes = data.convertToBytes()
            def result = mqttHandler.sendMessageToDevice(info.mac, bytes)
            if (!result) {
                return new Result(false, "信息发送失败")
            }
            Thread.sleep(10)
        }
        def commit = new AdminCommitPush(info.mac)
        return commitPush(commit, token)
    }

    @Override
    Result commitPush(AdminCommitPush info, String token) {
        def device = deviceRepository.findDeviceByMac(info.mac)
        if (device == null) {
            return new Result(false, "设备不存在")
        }
        def result = mqttHandler.sendMessageToDevice(info.mac, MqttConstant.CONFIRM_COMMIT)
        if (result) {
            return new Result(true, "数据已推送")
        }
        return new Result(false, "信息发送失败")
    }

    @Autowired
    @Resource
    void setMqttHandler(MQTTHandler mqttHandler) {
        this.mqttHandler = mqttHandler
    }

    @Autowired
    @Resource
    void setDeviceRepository(DeviceRepository deviceRepository) {
        this.deviceRepository = deviceRepository
    }

    @Autowired
    @Resource
    void setSpecificationFactory(SpecificationFactory specificationFactory) {
        this.specificationFactory = specificationFactory
    }

    @Autowired
    @Resource
    void setAreaRepository(AreaRepository areaRepository) {
        this.areaRepository = areaRepository
    }

    @Autowired
    @Resource
    void setCredentialRepository(CredentialRepository credentialRepository) {
        this.credentialRepository = credentialRepository
    }

    @Autowired
    @Resource
    void setSignInRecordRepository(SignInRecordRepository signInRecordRepository) {
        this.signInRecordRepository = signInRecordRepository
    }

    @Autowired
    @Resource
    void setRedisTemplate(RedisTemplate<String, Object> redisTemplate) {
        this.redisTemplate = redisTemplate
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setUserRepository(UserRepository userRepository) {
        this.userRepository = userRepository
    }

    @Autowired
    @Resource
    void setRelationshipRepository(DeviceRelationshipRepository relationshipRepository) {
        this.relationshipRepository = relationshipRepository
    }

    @Autowired
    @Resource
    void setFingerPrintDataRepository(FingerPrintDataRepository fingerPrintDataRepository) {
        this.fingerPrintDataRepository = fingerPrintDataRepository
    }

    @Autowired
    @Resource
    void setClassRepository(ClassRepository classRepository) {
        this.classRepository = classRepository
    }
}
