package org.brynhild.graduation.device.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.RedisConstant
import org.brynhild.graduation.common.utils.RandomCodeUtil
import org.brynhild.graduation.common.utils.TimeUtil
import org.brynhild.graduation.device.event.LogSender
import org.brynhild.graduation.device.factory.OperationLogFactory
import org.brynhild.graduation.device.factory.SystemLogFactory
import org.brynhild.graduation.device.service.IDeviceService
import org.brynhild.graduation.persistence.device.entity.SignInRecord
import org.brynhild.graduation.persistence.device.repository.CredentialRepository
import org.brynhild.graduation.persistence.device.repository.DeviceRepository
import org.brynhild.graduation.persistence.device.repository.SignInRecordRepository
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.transfer.device.bo.AuthenticRequest
import org.brynhild.graduation.transfer.device.bo.HeartBeatRequest
import org.brynhild.graduation.transfer.device.bo.RegisterRequest
import org.brynhild.graduation.transfer.device.bo.RegisterResponse
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.stereotype.Service

import java.util.concurrent.TimeUnit

@Service
class DeviceServiceImpl implements IDeviceService {
    private DeviceRepository deviceRepository
    private RedisTemplate<String, Object> redisTemplate
    private OperationLogFactory operationLogFactory
    private SystemLogFactory systemLogFactory
    private LogSender logSender
    private CredentialRepository credentialRepository
    private SignInRecordRepository signInRecordRepository


    @Override
    int authentic(AuthenticRequest request) {
        def credential = credentialRepository.findCredentialByDeviceMacAndUserCredential(request.mac, request.uid)
        if (credential == null || !credential.active) {
            return AuthenticRequest.hasError()
        }
        //TODO 一个Timetable里面只记录一次

        def signIn = new SignInRecord()
        signIn.with {
            it.userId = credential.userId
            it.areaId = credential.areaId
            it.deviceId = credential.deviceId
            it.userName = credential.userName
            it.areaName = credential.areaName
            it.deviceName = credential.deviceName
        }
        signInRecordRepository.save(signIn)
        def log = new OperationLog()
        log.with {
            it.operatorId = credential.deviceId
            it.operatorName = credential.deviceName
            it.targetId = credential.userId
            it.targetName = credential.userName
            it.description = "${targetId}-->${targetName}于${TimeUtil.getCurrentTimeString()}在${operatorId}-->${operatorName}（所属区域：${credential.areaName}）签到成功"
        }
        logSender.sendOperationLog(log)
        return AuthenticRequest.ok()
    }

    @Override
    int heartbeat(HeartBeatRequest request) {
        redisTemplate.opsForValue().set(RedisConstant.DEVICE_HEART_BEAT + request.mac, request.mac, 2, TimeUnit.MINUTES)
        return HeartBeatRequest.ok()
    }

    @Override
    RegisterResponse register(RegisterRequest registerRequest) {
        def device = deviceRepository.findDeviceByMac(registerRequest.mac)
        if (device == null) {
            def code = null
            def deviceInfo = null
            do {
                code = RandomCodeUtil.getRandomNumberCode(6)
                deviceInfo = redisTemplate.opsForValue().get(RedisConstant.DEVICE_CHECK_CODE)
            } while (deviceInfo != null)
            redisTemplate.opsForValue().set(RedisConstant.DEVICE_CHECK_CODE + code, registerRequest.mac, 10, TimeUnit.MINUTES)
            return new RegisterResponse(0, code)
        }
        if (!device.active) {
            return new RegisterResponse(0, "000000")
        }
        def origin = device.toString()
        device.setLastActive(TimeUtil.getCurrentTimeString())
        deviceRepository.save(device)
        def newData = device.toString()
        def log = systemLogFactory.generateLogWithDeviceRegister(device, origin, newData)
        logSender.sendSystemLog(log)
        return new RegisterResponse(1, "000000")
    }


    @Autowired
    @Resource
    void setDeviceRepository(DeviceRepository deviceRepository) {
        this.deviceRepository = deviceRepository
    }

    @Autowired
    @Resource
    void setRedisTemplate(RedisTemplate<String, Object> redisTemplate) {
        this.redisTemplate = redisTemplate
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }

    @Autowired
    @Resource
    void setSystemLogFactory(SystemLogFactory systemLogFactory) {
        this.systemLogFactory = systemLogFactory
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setCredentialRepository(CredentialRepository credentialRepository) {
        this.credentialRepository = credentialRepository
    }

    @Autowired
    @Resource
    void setSignInRecordRepository(SignInRecordRepository signInRecordRepository) {
        this.signInRecordRepository = signInRecordRepository
    }
}
