package org.brynhild.graduation.file.executor;

import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.auth.COSCredentials;
import com.qcloud.cos.http.HttpProtocol;
import com.qcloud.cos.model.PutObjectRequest;
import com.qcloud.cos.region.Region;
import com.qcloud.cos.transfer.Transfer;
import com.qcloud.cos.transfer.TransferManager;
import lombok.RequiredArgsConstructor;
import lombok.val;
import org.brynhild.graduation.file.config.CosConfig;
import org.brynhild.graduation.persistence.file.entity.LocalFile;
import org.brynhild.graduation.persistence.file.repository.FileRepository;
import org.brynhild.graduation.transfer.file.bo.UploadTask;
import org.springframework.stereotype.Repository;

import javax.annotation.PostConstruct;
import java.io.File;
import java.util.HashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;

@Repository
@RequiredArgsConstructor
public class CosJavaRepository {
    private final CosConfig config;
    private final FileRepository localFileRepository;
    private LinkedBlockingQueue<UploadTask> taskQueue = new LinkedBlockingQueue<>();
    private TransferManager transferManager;

    @PostConstruct
    public void init() {
        val protocolMap = new HashMap<String, HttpProtocol>() {{
            put("http", HttpProtocol.http);
            put("https", HttpProtocol.https);
            put("HTTP", HttpProtocol.http);
            put("HTTPS", HttpProtocol.https);
        }};
        if (protocolMap.get(config.getProtocol()) == null) {
            throw new IllegalArgumentException("Invalid HttpProtocol:" + config.getProtocol());
        }
        COSCredentials credential = new BasicCOSCredentials(config.getSecretId(), config.getSecretKey());
        Region region = new Region(config.getRegion());
        ClientConfig clientConfig = new ClientConfig(region);
        clientConfig.setHttpProtocol(protocolMap.get(config.getProtocol()));
        COSClient cosClient = new COSClient(credential, clientConfig);
        ExecutorService threadPool = Executors.newFixedThreadPool(32);
        transferManager = new TransferManager(cosClient, threadPool);
        System.out.println("[Success]: Successfully init CosRepository");
    }

    public synchronized void addUploadFileTask(File targetFile, LocalFile localFile) {
        val empty = taskQueue.isEmpty();
        taskQueue.add(new UploadTask(targetFile, localFile));
        if (empty) {
            new Thread(() -> {
                while (!taskQueue.isEmpty()) {
                    try {
                        val task = taskQueue.take();
                        val file = task.getLocalFile();
                        uploadFile(targetFile, file.getPath());
                        file.setUpload(true);
                        localFileRepository.save(file);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }).start();
        }
    }

    public synchronized void addUploadFileTaskSync(File targetFile, LocalFile localFile) {
        uploadFile(targetFile, localFile.getPath());
        localFile.setUpload(true);
        localFileRepository.save(localFile);
    }

    private void uploadFile(File file, String key) {
        val putObjectRequest = new PutObjectRequest(config.getBucketName(), key, file);
        val upload = transferManager.upload(putObjectRequest);
        showTransferProgress(upload);
        try {
            val uploadResult = upload.waitForUploadResult();
            System.out.println(uploadResult.getETag());
            System.out.println(uploadResult.getCrc64Ecma());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void showTransferProgress(Transfer transfer) {
        System.out.println(transfer.getDescription());
        do {
            try {
                Thread.sleep(1000);
            } catch (Exception e) {
                e.printStackTrace();
                return;
            }
            val progress = transfer.getProgress();
            System.out.println("[" + progress.getBytesTransferred() + " / " + progress.getTotalBytesToTransfer() + "] = " + progress.getPercentTransferred());
        } while (!transfer.isDone());
        System.out.println(transfer.getState());
    }
}
