package org.brynhild.graduation.timetable.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.timetable.entity.Course
import org.brynhild.graduation.persistence.timetable.repository.CourseRepository
import org.brynhild.graduation.persistence.user.repository.AcademyRepository
import org.brynhild.graduation.timetable.event.LogSender
import org.brynhild.graduation.timetable.factory.OperationLogFactory
import org.brynhild.graduation.timetable.factory.SpecificationFactory
import org.brynhild.graduation.timetable.service.ICourseService
import org.brynhild.graduation.transfer.timetable.vo.AdminAddCourse
import org.brynhild.graduation.transfer.timetable.vo.AdminDeleteCourse
import org.brynhild.graduation.transfer.timetable.vo.AdminModifyCourse
import org.brynhild.graduation.transfer.timetable.vo.QueryCourseRequest
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class CourserServiceImpl implements ICourseService {
    private LogSender logSender
    private JwtUtil jwtUtil
    private CourseRepository courseRepository
    private AcademyRepository academyRepository

    private SpecificationFactory specificationFactory
    private OperationLogFactory operationLogFactory

    @Override
    Result addCourse(AdminAddCourse info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def optional = academyRepository.findById(info.department)
        if (optional.isEmpty()) {
            throw new IllegalArgumentException("删除失败，无相关数据")
        }
        def course = new Course()
        def log = operationLogFactory.generateLogWithAdminAddCourse(info, course)
        courseRepository.save(course)
        log.operatorName = name
        log.operatorId = id
        log.targetName = info.name
        log.targetId = course.id
        logSender.sendOperationLog(log)

        return new Result(true, "添加成功")
    }

    @Override
    Result findCourse(QueryCourseRequest info) {
        def specification = specificationFactory.getCourseSpecification(info)
        println courseRepository.findAll(specification)
        return new Result(true, "查询成功", courseRepository.findAll(specification))
    }

    @Override
    Result modifyCourse(AdminModifyCourse info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def optional = courseRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "目标不存在")
        }
        def course = optional.get()
        def log = operationLogFactory.generateLogWithAdminModifyCourse(info, course)
        courseRepository.save(course)
        log.operatorName = name
        log.operatorId = id
        log.targetName = info.name
        log.targetId = course.id
        logSender.sendOperationLog(log)

        return new Result(true, "修改成功")
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result deleteCourse(AdminDeleteCourse info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> logs = new ArrayList<>()
        for (def cid in info.id) {
            println cid
            def optional = courseRepository.findById(cid)
            if (optional.isEmpty()) {
                return new Result(false, "删除失败，无相关数据")
            }
            def course = optional.get()
            def log = new OperationLog()
            log.operatorName = name
            log.operatorId = id
            log.targetName = course.name
            log.targetId = course.id
            log.description = "管理员${id}->${name}删除了课程${course.id}->${course.name}"
            courseRepository.logicDelete(course.id)
            logs.add(log)
        }

        for (def log in logs) {
            logSender.sendOperationLog(log)
        }

        return new Result(true, "删除成功")
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setCourseRepository(CourseRepository courseRepository) {
        this.courseRepository = courseRepository
    }

    @Autowired
    @Resource
    void setSpecificationFactory(SpecificationFactory specificationFactory) {
        this.specificationFactory = specificationFactory
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }

    @Autowired
    @Resource
    void setAcademyRepository(AcademyRepository academyRepository) {
        this.academyRepository = academyRepository
    }
}
