package org.brynhild.graduation.timetable.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.persistence.device.repository.AreaRepository
import org.brynhild.graduation.persistence.device.repository.SignInRecordRepository
import org.brynhild.graduation.persistence.file.entity.LocalFile
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.timetable.entity.Course
import org.brynhild.graduation.persistence.timetable.entity.TimeTable
import org.brynhild.graduation.persistence.timetable.repository.CourseRepository
import org.brynhild.graduation.persistence.timetable.repository.TimeTableRepository
import org.brynhild.graduation.persistence.user.repository.ClassRepository
import org.brynhild.graduation.persistence.user.repository.UserRepository
import org.brynhild.graduation.timetable.config.CosConfig
import org.brynhild.graduation.timetable.event.LogSender
import org.brynhild.graduation.timetable.executor.CosJavaRepository
import org.brynhild.graduation.timetable.executor.WordExecutor
import org.brynhild.graduation.timetable.factory.OperationLogFactory
import org.brynhild.graduation.timetable.factory.SpecificationFactory
import org.brynhild.graduation.timetable.service.ITimeTableService
import org.brynhild.graduation.transfer.file.dto.FileInfo
import org.brynhild.graduation.transfer.timetable.bo.ClassStatisticInfo
import org.brynhild.graduation.transfer.timetable.bo.CourseStatisticInfo
import org.brynhild.graduation.transfer.timetable.bo.StudentStatisticInfo
import org.brynhild.graduation.transfer.timetable.vo.*
import org.brynhild.graduation.transfer.view.ViewConvertor
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

import java.util.stream.Collectors

@Service
class TimeTableServiceImpl implements ITimeTableService {
    private LogSender logSender
    private JwtUtil jwtUtil
    private CourseRepository courseRepository
    private TimeTableRepository tableRepository
    private AreaRepository areaRepository
    private SpecificationFactory specificationFactory
    private OperationLogFactory operationLogFactory
    private UserRepository userRepository
    private SignInRecordRepository signInRecordRepository
    private ClassRepository classRepository
    private CosJavaRepository cosJavaRepository
    private ViewConvertor convertor
    private CosConfig config

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result addTimeTable(AdminAddTimeTable info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def areaOptional = areaRepository.findById(info.areaId)
        def courseOptional = courseRepository.findById(info.courseId)
        if (areaOptional.isEmpty() || courseOptional.isEmpty()) {
            return new Result(false, "课程或教室不存在")
        }

        def weeks = info.week
        List<OperationLog> logs = new ArrayList<>()

        def course = courseOptional.get()
        info.week.split(",").toList().stream().map { it -> Integer.parseInt(it) }.forEachOrdered {
            def timeTable = new TimeTable()
            def log = operationLogFactory.generateLogWithAdminAddTimeTable(info, timeTable)
            timeTable.week = it
            timeTable.courseName = course.name
            println timeTable
            tableRepository.save(timeTable)
            log.operatorName = name
            log.operatorId = id
            log.targetName = "${timeTable.id}"
            log.targetId = timeTable.id
        }

        for (log in logs) {
            logSender.sendOperationLog(log)
        }

        return new Result(true, "添加成功")
    }

    @Override
    Result findTimeTable(QueryTimeTableRequest info) {
        println info
        def specification = specificationFactory.getTimeTableSpecification(info)
        def listOfTables = tableRepository.findAll(specification)
        println listOfTables
        def tableInfos = convertor.convertToTimeTableInfo(listOfTables)
        return new Result(true, "查询成功", tableInfos)
    }

    @Override
    Result modifyTimeTable(AdminModifyTimeTable info, String token) {
        return null
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result deleteTimeTable(AdminDeleteTimeTable info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> logs = new ArrayList<>()

        for (tid in info.id) {
            def optional = tableRepository.findById(tid)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("删除失败，无相关数据")
            }
            def timeTable = optional.get()
            def log = new OperationLog()
            log.operatorName = name
            log.operatorId = id
            log.targetName = timeTable.courseName
            log.targetId = timeTable.id
            log.description = "管理员${id}->${name}删除了课程安排${timeTable.id}->${timeTable.courseName}"
            logs.add(log)
            courseRepository.logicDelete(timeTable.id)
        }

        for (log in logs) {
            logSender.sendOperationLog(log)
        }

        return new Result(true, "删除成功")
    }

    @Override
    Result queryClassStatistic(AdminQueryClassStatistic info, String token) {
        def result = getClassStatistic(info)
        return new Result(true, "查询成功", result)
    }

    private List<ClassStatisticInfo> getClassStatistic(AdminQueryClassStatistic info) {
        def timeTableByClass = tableRepository.findTimeTableByClazz("${info.id}")
        def courseMap = new HashMap<Long, Course>()
        def countMap = new HashMap<Long, Long>()
        def actualCountMap = new HashMap<Long, Long>()

        def courseIdSet = timeTableByClass.stream().map { it.courseId }.collect(Collectors.toSet())
        for (courseId in courseIdSet) {
            courseRepository.findById(courseId).ifPresent { courseMap.put(courseId, it) }
        }
        timeTableByClass.stream().forEachOrdered {
            countMap.computeIfAbsent(it.courseId, i -> 0L)
            countMap.put(it.courseId, countMap.get(it.courseId) + 1L)
        }

        def listOfUsers = userRepository.findUserByDepartmentId(info.id)
        def userCount = listOfUsers.size()
        List<ClassStatisticInfo> infoList = new ArrayList<>()

        println "${listOfUsers.size()},${courseIdSet.size()}"

        for (timetable in timeTableByClass) {
            def startSignInTime = timetable.getStartSignInTime()
            def endSignInTime = timetable.getEndSignInTime()

            def startSignOutTime = timetable.getStartSignOutTime()
            def endSignOutTime = timetable.getEndSignOutTime()

            println "${startSignInTime}----${endSignInTime}"
            println "${startSignOutTime}----${endSignOutTime}"

            def listOfRecords = signInRecordRepository.findSignInRecordByTimeBetweenAndAreaId(startSignInTime, endSignInTime, timetable.areaId)
            def signIn = listOfRecords.stream().map { it.userId }.collect(Collectors.toSet())
            listOfRecords = signInRecordRepository.findSignInRecordByTimeBetweenAndAreaId(timetable.startSignOutTime, timetable.endSignOutTime, timetable.areaId)
            def signOut = listOfRecords.stream().map { it.userId }.collect(Collectors.toSet())
            signIn.retainAll(signOut)

            actualCountMap.computeIfAbsent(timetable.courseId, i -> 0L)
            actualCountMap.put(timetable.courseId, actualCountMap.get(timetable.courseId) + signIn.size())
        }

        for (course in courseMap.values()) {
            infoList.add(new ClassStatisticInfo(course, countMap.get(course.id) * userCount, actualCountMap.get(course.id)))
        }

        infoList = infoList.sort()
        return infoList
    }


    @Override
    Result queryStudentStatistic(AdminQueryStudentStatistic info, String token) {
        def result = getStudentStatistic(info)
        return new Result(true, "查询成功", result)
    }

    private List<StudentStatisticInfo> getStudentStatistic(AdminQueryStudentStatistic info) {
        def optional = userRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new ArrayList<StudentStatisticInfo>()
        }
        def user = optional.get()
        def timeTableByClass = tableRepository.findTimeTableByClazz("${user.departmentId}")
        def courseMap = new HashMap<Long, Course>()
        def countMap = new HashMap<Long, Long>()
        def actualCountMap = new HashMap<Long, Long>()

        def courseIdSet = timeTableByClass.stream().map { it.courseId }.collect(Collectors.toSet())
        for (courseId in courseIdSet) {
            courseRepository.findById(courseId).ifPresent { courseMap.put(courseId, it) }
        }
        timeTableByClass.stream().forEach {
            countMap.computeIfAbsent(it.courseId, i -> 0L)
//            countMap.computeIfPresent(it.courseId,i->i+1)
            countMap.put(it.courseId, countMap.get(it.courseId) + 1)
        }

        List<StudentStatisticInfo> infoList = new ArrayList<>()

        for (timetable in timeTableByClass) {
            def startSignInTime = timetable.getStartSignInTime()
            def endSignInTime = timetable.getEndSignInTime()

            def listOfRecords = signInRecordRepository.findSignInRecordByTimeBetweenAndAreaIdAndUserId(startSignInTime, endSignInTime, timetable.areaId, info.id)
            if (listOfRecords.isEmpty()) {
                continue
            }
            listOfRecords = signInRecordRepository.findSignInRecordByTimeBetweenAndAreaIdAndUserId(timetable.startSignOutTime, timetable.endSignOutTime, timetable.areaId, info.id)
            if (listOfRecords.isEmpty()) {
                continue
            }

            actualCountMap.computeIfAbsent(timetable.courseId, i -> 0L)
//            actualCountMap.computeIfPresent(timetable.courseId,i->i+signIn.size())
            actualCountMap.put(timetable.courseId, actualCountMap.get(timetable.courseId) + 1)
        }

        for (course in courseMap.values()) {
            infoList.add(new StudentStatisticInfo(course, countMap.get(course.id), actualCountMap.getOrDefault(course.id, 0L)))
        }

        infoList = infoList.sort()
        return infoList
    }

    @Override
    Result generateClassStatistic(AdminQueryClassStatistic info, String token) {
        def optional = classRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "查询失败");
        }
        def result = getClassStatistic(info)
        def filename = "docOutput/class-${info.id}.docx"
        def map = new HashMap<String, String>()
        def clazz = optional.get()
        map.put("title", clazz.name)
        WordExecutor.generateClassStatisticReport(filename, result, map)
        def localFile = new LocalFile();
        localFile.path = "class-${info.id}.docx"
        cosJavaRepository.addUploadFileTaskSync(new File(filename), localFile)

        FileInfo fileInfo = new FileInfo();
        fileInfo.setPath(config.getRedirectPath() + localFile.getPath());
        def list = new ArrayList<FileInfo>()
        list.add(fileInfo)

        return new Result(true, "生成成功", list)
    }

    @Override
    Result generateStudentStatistic(AdminQueryStudentStatistic info, String token) {
        def optional = userRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "查询失败")
        }
        def user = optional.get()
        def result = getStudentStatistic(info)
        def filename = "docOutput/student-${info.id}.docx"
        def map = new HashMap<String, String>()
        map.put("title", user.name)
        WordExecutor.generateStudentStatisticReport(filename, result, map)
        def localFile = new LocalFile();

        localFile.path = "student-${info.id}.docx"
        cosJavaRepository.addUploadFileTaskSync(new File(filename), localFile)

        FileInfo fileInfo = new FileInfo();
        fileInfo.setPath(config.getRedirectPath() + localFile.getPath());
        def list = new ArrayList<FileInfo>()
        list.add(fileInfo)
        return new Result(true, "生成成功", list)
    }

    @Override
    Result queryTeachingClass(QueryTeachingClassRequest request) {
        def listOfTables = tableRepository.findTimeTableByTeacher(request.getTeacherId())
        def collect = listOfTables.stream().map { Long.parseLong(it.clazz) }.collect(Collectors.toSet())
        def classes = classRepository.findAllById(collect)
        return new Result(true, "查询成功", classes)
    }

    private List<CourseStatisticInfo> getTeachingCourseStatistic(QueryTeachingCourseStatisticRequest request) {
        def timeTableByClass = tableRepository.findTimeTableByTeacher(request.id)
        def courseMap = new HashMap<Long, Course>()
        def countMap = new HashMap<Long, Long>()
        def actualCountMap = new HashMap<Long, Long>()

        def courseIdSet = timeTableByClass.stream().map { it.courseId }.collect(Collectors.toSet())
        for (courseId in courseIdSet) {
            courseRepository.findById(courseId).ifPresent { courseMap.put(courseId, it) }
        }
        timeTableByClass.stream().forEach {
            countMap.computeIfAbsent(it.courseId, i -> 0L)
            countMap.put(it.courseId, countMap.get(it.courseId) + 1)
        }
        List<CourseStatisticInfo> infoList = new ArrayList<>()

        for (timetable in timeTableByClass) {
            def startSignInTime = timetable.getStartSignInTime()
            def endSignInTime = timetable.getEndSignInTime()

            def startSignOutTime = timetable.getStartSignOutTime()
            def endSignOutTime = timetable.getEndSignOutTime()

            println "${startSignInTime}----${endSignInTime}"
            println "${startSignOutTime}----${endSignOutTime}"

            def listOfRecords = signInRecordRepository.findSignInRecordByTimeBetweenAndAreaId(startSignInTime, endSignInTime, timetable.areaId)
            def signIn = listOfRecords.stream().map { it.userId }.collect(Collectors.toSet())
            listOfRecords = signInRecordRepository.findSignInRecordByTimeBetweenAndAreaId(timetable.startSignOutTime, timetable.endSignOutTime, timetable.areaId)
            def signOut = listOfRecords.stream().map { it.userId }.collect(Collectors.toSet())
            signIn.retainAll(signOut)

            actualCountMap.computeIfAbsent(timetable.courseId, i -> 0L)
            actualCountMap.put(timetable.courseId, actualCountMap.get(timetable.courseId) + signIn.size())
        }

        for (course in courseMap.values()) {
            infoList.add(new CourseStatisticInfo(course, countMap.get(course.id), actualCountMap.getOrDefault(course.id, 0L)))
        }

        infoList = infoList.sort()
        return infoList
    }

    @Override
    Result queryTeachingCourseStatistic(QueryTeachingCourseStatisticRequest request) {
        def optional = userRepository.findById(request.id)
        if (optional.isEmpty()) {
            return new Result(false, "用户不存在")
        }
        return new Result(true, "查询成功", getTeachingCourseStatistic(request))
    }

    @Override
    Result generateTeachingCourseStatistic(QueryTeachingCourseStatisticRequest request) {
        def optional = userRepository.findById(request.id)
        if (optional.isEmpty()) {
            return new Result(false, "用户不存在")
        }
        def user = optional.get()
        def result = getTeachingCourseStatistic(request)
        def filename = "docOutput/teaching-class-${request.id}.docx"
        def map = new HashMap<String, String>()
        map.put("title", "${user.name}教学课程")
        WordExecutor.generateTeachingCourseStatisticReport(filename, result, map)
        def localFile = new LocalFile();
        localFile.path = "teaching-class-${request.id}.docx"
        cosJavaRepository.addUploadFileTaskSync(new File(filename), localFile)

        FileInfo fileInfo = new FileInfo();
        fileInfo.setPath(config.getRedirectPath() + localFile.getPath());
        def list = new ArrayList<FileInfo>()
        list.add(fileInfo)

        return new Result(true, "生成成功", list)
    }

    @Override
    Result deleteTimeTable(AdminDeleteTimeTableByDetail info, String token) {
        def listOfTables = tableRepository.findTimeTableByWeekAndDayAndIndexAndClazz("${info.week}", info.day, info.index, info.clazz)
        for(def table:listOfTables){
            tableRepository.delete(table)
        }
        return new Result(true,"删除成功")
    }

    @Autowired
    @Resource
    void setConfig(CosConfig config) {
        this.config = config
    }

    @Autowired
    @Resource
    void setCosJavaRepository(CosJavaRepository cosJavaRepository) {
        this.cosJavaRepository = cosJavaRepository
    }

    @Autowired
    @Resource
    void setClassRepository(ClassRepository classRepository) {
        this.classRepository = classRepository
    }

    @Autowired
    @Resource
    void setSignInRecordRepository(SignInRecordRepository signInRecordRepository) {
        this.signInRecordRepository = signInRecordRepository
    }

    @Autowired
    @Resource
    void setUserRepository(UserRepository userRepository) {
        this.userRepository = userRepository
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setCourseRepository(CourseRepository courseRepository) {
        this.courseRepository = courseRepository
    }

    @Autowired
    @Resource
    void setTableRepository(TimeTableRepository tableRepository) {
        this.tableRepository = tableRepository
    }

    @Autowired
    @Resource
    void setSpecificationFactory(SpecificationFactory specificationFactory) {
        this.specificationFactory = specificationFactory
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }

    @Autowired
    @Resource
    void setAreaRepository(AreaRepository areaRepository) {
        this.areaRepository = areaRepository
    }

    @Autowired
    @Resource
    void setConvertor(ViewConvertor convertor) {
        this.convertor = convertor
    }
}
