package org.brynhild.graduation.user.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.user.entity.Academy
import org.brynhild.graduation.persistence.user.entity.Class
import org.brynhild.graduation.persistence.user.entity.Major
import org.brynhild.graduation.persistence.user.entity.User
import org.brynhild.graduation.persistence.user.repository.AcademyRepository
import org.brynhild.graduation.persistence.user.repository.ClassRepository
import org.brynhild.graduation.persistence.user.repository.MajorRepository
import org.brynhild.graduation.persistence.user.repository.UserRepository
import org.brynhild.graduation.transfer.user.vo.*
import org.brynhild.graduation.user.event.LogSender
import org.brynhild.graduation.user.factory.OperationLogFactory
import org.brynhild.graduation.user.factory.SpecificationFactory
import org.brynhild.graduation.user.service.IAdminService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminServiceImpl implements IAdminService {
    private UserRepository userRepository
    private AcademyRepository academyRepository
    private ClassRepository classRepository
    private MajorRepository majorRepository
    private JwtUtil jwtUtil
    private OperationLogFactory operationLogFactory
    private LogSender logSender
    private SpecificationFactory specificationFactory

    @Override
    Result findUser(QueryUserRequest info, String token) {
        def userList = new ArrayList<User>()
        if (info.id != null) {
            def optional = userRepository.findById(info.id)
            if (optional.isPresent()) {
                userList.add(optional.get())
                return new Result(true, "查询成功", userList)
            } else {
                return new Result(false, "无相关用户")
            }
        }
        def specification = info.getSpecification()
        return new Result(true, "查询成功", userRepository.findAll(specification))
    }

    @Override
    Result addUser(AdminAddUser info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        def user = userRepository.findUserByUsernameOrNo(info.username, info.no)
        if (user != null) {
            return new Result(false, "重复添加")
        }

        user = new User()
        def log = operationLogFactory.generateLogWithAdminAddUser(info, user)
        user = userRepository.save(user)
        log.operatorName = name
        log.operatorId = id
        log.targetName = info.name
        log.targetId = user.id

        logSender.sendOperationLog(log)
        return new Result(true, "添加成功", user)
    }

    @Override
    Result modifyUser(AdminModifyUser info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        def optional = userRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "查询失败，无此用户")
        }
        def user = optional.get()
        def log = operationLogFactory.generateLogWithAdminModifyUser(info, user)
        userRepository.save(user)
        log.operatorName = name
        log.operatorId = id
        log.targetName = user.name
        log.targetId = user.id

        logSender.sendOperationLog(log)

        return new Result(true, "修改成功", user)
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result deleteUser(AdminDeleteUser info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> operationLogList = new ArrayList<>()
        for (Long userId : info.id) {
            def optional = userRepository.findById(userId)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("没有这个用户")
            }
            def user = optional.get()
            def log = new OperationLog()
            log.operatorName = name
            log.operatorId = id
            log.targetName = user.name
            log.targetId = user.id
            log.description = "管理员${id}->${name}删除了用户${user.id}->${user.name}"
            userRepository.logicDelete(user.id)
            operationLogList.add(log)
        }

        for (OperationLog log : operationLogList) {
            logSender.sendOperationLog(log)
        }


        return new Result(true, "删除成功")
    }

    @Override
    Result findAcademy(QueryAcademyRequest info) {
        def specification = specificationFactory.getAcademySpecification(info)
        return new Result(true, "查询成功", academyRepository.findAll(specification))
    }

    @Override
    Result addAcademy(AdminAddAcademy info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def academy = academyRepository.findAcademyByNameOrNo(info.name, info.no)
        if (academy != null) {
            return new Result(false, "重复添加")
        }
        academy = new Academy()
        def log = operationLogFactory.generateLogWithAdminAddAcademy(info, academy)
        academyRepository.save(academy)
        log.operatorName = name
        log.operatorId = id
        log.targetName = info.name
        log.targetId = academy.id

        logSender.sendOperationLog(log)
        return new Result(true, "添加成功")
    }

    @Override
    Result modifyAcademy(AdminModifyAcademy info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        def optional = academyRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "查询失败，无相关数据")
        }
        def academy = optional.get()
        def log = operationLogFactory.generateLogWithAdminModifyAcademy(info, academy)
        academyRepository.save(academy)
        log.operatorName = name
        log.operatorId = id
        log.targetName = academy.name
        log.targetId = academy.id

        logSender.sendOperationLog(log)

        return new Result(true, "修改成功", academy)
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result deleteAcademy(AdminDeleteAcademy info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> logList = new ArrayList<>()
        for (Long acaId : info.id) {
            def optional = academyRepository.findById(acaId)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("没有这个学院")
            }
            def academy = optional.get()
            def log = new OperationLog()
            log.operatorName = name
            log.operatorId = id
            log.targetName = academy.name
            log.targetId = academy.id
            log.description = "管理员${id}->${name}删除了学院${academy.id}->${academy.name}"
            academyRepository.logicDelete(academy.id)
            logList.add(log)
        }

        for (OperationLog log : logList) {
            logSender.sendOperationLog(log)
        }
        return new Result(true, "删除成功")
    }

    @Override
    Result findMajor(QueryMajorRequest info) {
        return new Result(true, "查询成功", majorRepository.findAll(specificationFactory.getMajorSpecification(info)))
    }

    @Override
    Result addMajor(AdminAddMajor info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def major = majorRepository.findMajorByNameOrCode(info.name, info.code)
        if (major != null) {
            return new Result(false, "重复添加")
        }
        major = new Major()
        def log = operationLogFactory.generateLogWithAdminAddMajor(info, major)
        majorRepository.save(major)
        log.operatorName = name
        log.operatorId = id
        log.targetName = info.name
        log.targetId = major.id

        logSender.sendOperationLog(log)
        return new Result(true, "添加成功")
    }

    @Override
    Result modifyMajor(AdminModifyMajor info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        def optional = majorRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "查询失败，无相关数据")
        }
        def major = optional.get()
        def log = operationLogFactory.generateLogWithAdminModifyMajor(info, major)
        majorRepository.save(major)
        log.operatorName = name
        log.operatorId = id
        log.targetName = major.name
        log.targetId = major.id

        logSender.sendOperationLog(log)

        return new Result(true, "修改成功", major)
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result deleteMajor(AdminDeleteMajor info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> logList = new ArrayList<>()
        for (Long acaId : info.id) {
            def optional = majorRepository.findById(acaId)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("没有这个专业")
            }
            def major = optional.get()
            def log = new OperationLog()
            log.operatorName = name
            log.operatorId = id
            log.targetName = major.name
            log.targetId = major.id
            log.description = "管理员${id}->${name}删除了专业${major.id}->${major.name}"
            majorRepository.logicDelete(major.id)
            logList.add(log)
        }

        for (OperationLog log : logList) {
            logSender.sendOperationLog(log)
        }
        return new Result(true, "删除成功")
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    Result deleteClass(AdminDeleteClass info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        List<OperationLog> logList = new ArrayList<>()
        for (Long acaId : info.id) {
            def optional = classRepository.findById(acaId)
            if (optional.isEmpty()) {
                throw new IllegalArgumentException("没有这个班级")
            }
            def clazz = optional.get()
            def log = new OperationLog()
            log.operatorName = name
            log.operatorId = id
            log.targetName = clazz.name
            log.targetId = clazz.id
            log.description = "管理员${id}->${name}删除了班级${clazz.id}->${clazz.name}"
            classRepository.logicDelete(clazz.id)
            logList.add(log)
        }

        for (OperationLog log : logList) {
            logSender.sendOperationLog(log)
        }
        return new Result(true, "删除成功")
    }

    @Override
    Result modifyClass(AdminModifyClass info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)

        def optional = classRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "查询失败，无相关数据")
        }
        def clazz = optional.get()
        def log = operationLogFactory.generateLogWithAdminModifyClass(info, clazz)
        classRepository.save(clazz)
        log.operatorName = name
        log.operatorId = id
        log.targetName = clazz.name
        log.targetId = clazz.id

        logSender.sendOperationLog(log)

        return new Result(true, "修改成功", clazz)
    }

    @Override
    Result addClass(AdminAddClass info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.get(AccountConstant.TOKEN_ID) as String)
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def clazz = classRepository.findClassByNameOrNo(info.name, info.no)
        if (clazz != null) {
            return new Result(false, "重复添加")
        }
        clazz = new Class()
        def log = operationLogFactory.generateLogWithAdminAddClass(info, clazz)
        classRepository.save(clazz)
        log.operatorName = name
        log.operatorId = id
        log.targetName = info.name
        log.targetId = clazz.id

        logSender.sendOperationLog(log)
        return new Result(true, "添加成功")
    }

    @Override
    Result findClass(QueryClassRequest info) {
        return new Result(true, "查询成功", classRepository.findAll(specificationFactory.getClassSpecification(info)))
    }

    @Autowired
    @Resource
    void setUserRepository(UserRepository userRepository) {
        this.userRepository = userRepository
    }

    @Autowired
    @Resource
    void setAcademyRepository(AcademyRepository academyRepository) {
        this.academyRepository = academyRepository
    }

    @Autowired
    @Resource
    void setClassRepository(ClassRepository classRepository) {
        this.classRepository = classRepository
    }

    @Autowired
    @Resource
    void setMajorRepository(MajorRepository majorRepository) {
        this.majorRepository = majorRepository
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setSpecificationFactory(SpecificationFactory specificationFactory) {
        this.specificationFactory = specificationFactory
    }
}
