package org.brynhild.graduation.user.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.*
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.common.utils.StringUtil
import org.brynhild.graduation.common.utils.TimeUtil
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.user.entity.User
import org.brynhild.graduation.persistence.user.repository.UserRepository
import org.brynhild.graduation.transfer.user.bo.SendActiveEmailEvent
import org.brynhild.graduation.transfer.user.bo.SendResetEmailEvent
import org.brynhild.graduation.transfer.user.bo.UserSuccessLoginEvent
import org.brynhild.graduation.transfer.user.dto.UserInfo
import org.brynhild.graduation.transfer.user.vo.LoginRequest
import org.brynhild.graduation.transfer.user.vo.RegisterUser
import org.brynhild.graduation.transfer.user.vo.ResendEmail
import org.brynhild.graduation.transfer.user.vo.ResetPassword
import org.brynhild.graduation.user.event.LogSender
import org.brynhild.graduation.user.factory.OperationLogFactory
import org.brynhild.graduation.user.service.IUserService
import org.springframework.amqp.rabbit.core.RabbitTemplate
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.stereotype.Service

import java.util.concurrent.TimeUnit

@Service
class UserServiceImpl implements IUserService {

    private UserRepository userRepository
    private RedisTemplate<String, Object> redisTemplate
    private RabbitTemplate rabbitTemplate
    private BCryptPasswordEncoder encoder
    private JwtUtil jwtUtil
    private LogSender logSender
    private OperationLogFactory operationLogFactory

    @Override
    Result register(RegisterUser info) {
        def user = userRepository.findUserByUsernameOrNo(info.username, info.no)
        if (user != null) {
            return new Result(false, "用户名或编号已被注册")
        }
        info.password = encoder.encode(info.password)

        user = new User()
        def log = operationLogFactory.generateLogWithRegisterUser(info, user)
        userRepository.save(user)
        redisRecordEmailToUser(user)

        def sendEmail = new SendActiveEmailEvent(info.email, user.hashCode() + "")
        rabbitTemplate.convertAndSend(RabbitQueueConstant.EMAIL, ByteTransferUtil.getBytesFromObject(sendEmail))

        log.targetName = log.operatorName = info.name
        log.targetId = log.operatorId = user.id
        logSender.sendOperationLog(log)

        return new Result(true, "注册成功，请验证电子邮件地址后登录，验证邮件10分钟内有效")
    }

    @Override
    Result active(String hash) {
        def userObject = redisTemplate.opsForValue().get(RedisConstant.ACCOUNT_WAIT_CONFIRM + hash)
        if (userObject == null) {
            return new Result(false, "激活失败，请填写信息来重发激活邮件")
        }
        def user = (User) userObject
        user = userRepository.findUserByUsername(user.username)
        if (user == null) {
            return new Result(false, "账户异常，请联系管理员")
        }
        user.setState(UserStateConstant.READY)
        userRepository.save(user)
        redisDeleteUserEmail(user, hash)

        def log = new OperationLog()
        log.targetId = log.operatorId = user.id
        log.operatorName = log.targetName = user.name
        log.description = "用户：${user.name}通过验证邮件地址激活了账户"
        logSender.sendOperationLog(log)

        return new Result(true, "激活成功")
    }

    @Override
    Result reset(ResetPassword info) {
        def userObject = redisTemplate.opsForValue().get(RedisConstant.ACCOUNT_WAIT_CONFIRM + info.hash)
        if (userObject == null) {
            return new Result(false, "验证失败，请填写信息来重发激活邮件")
        }
        if (StringUtil.isEmpty(info.password)) {
            return new Result(true, "信息有效")
        }

        def user = (User) userObject
        user = userRepository.findUserByUsername(user.username)
        if (user == null) {
            return new Result(false, "账户异常，请联系管理员")
        }
        user.password = encoder.encode(info.password)
        userRepository.save(user)
        redisDeleteUserEmail(user, info.hash)

        def log = new OperationLog()
        log.with {
            log.targetId = log.operatorId = user.id
            log.operatorName = log.targetName = user.name
            log.description = "用户：${user.name}通过验证邮件地址修改了账户密码"
        }
        logSender.sendOperationLog(log)
        return new Result(true, "激活成功")

    }

    @Override
    Result resend(ResendEmail info) {
        def emailRecord = redisTemplate.opsForValue().get(RedisConstant.ACCOUNT_WAIT_CONFIRM_EMAIL + info.email)
        if (emailRecord != null) {
            return new Result(false, "获取验证码频繁，请稍后再试")
        }
        def usernameRecord = redisTemplate.opsForValue().get(RedisConstant.ACCOUNT_WAIT_CONFIRM_USERNAME + info.username)
        if (usernameRecord != null) {
            return new Result(false, "获取验证码频繁，请稍后再试")
        }

        def user = userRepository.findUserByUsernameAndEmail(info.username, info.email)
        if (user == null) {
            return new Result(false, "用户不存在")
        }
        redisRecordEmailToUser(user)

        def sendEmail = null
        println info.type
        if (info.type == EmailConstant.ACTIVE_EMAIL) {
            sendEmail = new SendActiveEmailEvent(info.email, user.hashCode() + "")
        }
        if (info.type == EmailConstant.RESET_EMAIL) {
            sendEmail = new SendResetEmailEvent(info.email, user.hashCode() + "")
        }
        if (sendEmail == null) {
            return new Result(false, "参数有误")
        }

        rabbitTemplate.convertAndSend(RabbitQueueConstant.EMAIL, ByteTransferUtil.getBytesFromObject(sendEmail))
        return new Result(true, "验证邮件重发送成功，请验证电子邮件地址后登录，验证邮件10分钟内有效")
    }

    @Override
    Result login(LoginRequest info) {
        def user = userRepository.findUserByUsername(info.username)
        if (user == null) {
            return new Result(false, "用户不存在")
        }
        if (user.state != UserStateConstant.READY) {
            if (user.state == UserStateConstant.NOT_ACTIVE) {
                ResendEmail resendEmail = new ResendEmail()
                resendEmail.with {
                    type = EmailConstant.ACTIVE_EMAIL
                    email = user.email
                    username = info.username
                }
                return resend(resendEmail)
            }
            if (user.state == UserStateConstant.FORBIDDEN) {
                return new Result(false, '该账户已被禁用，请联系管理员', user.state)
            }
            throw new IllegalStateException()
        }
        if (!encoder.matches(info.password, user.password)) {
            return new Result(false, "用户名或密码错误")
        }
        def claims = new HashMap<String, Object>()
        claims.with {
            put(AccountConstant.TOKEN_ROLE, user.type + "")
            put(AccountConstant.TOKEN_NAME, user.name)
            put(AccountConstant.TOKEN_ID, user.id + "")
            put(AccountConstant.TOKEN_USERNAME, user.username)
        }
        def token = jwtUtil.createJwt(claims)

        user.lastLogin = TimeUtil.getCurrentTimeString()
        userRepository.save(user)

        def event = new UserSuccessLoginEvent(user.username, token)
        rabbitTemplate.convertAndSend(RabbitQueueConstant.USER, ByteTransferUtil.getBytesFromObject(event))

        def log = new OperationLog()
        log.operatorName = log.targetName = user.name
        log.operatorId = log.targetId = user.id
        log.description = "用户${user.name}登录成功"
        logSender.sendOperationLog(log)

        return new Result(true, "登录成功", new UserInfo(user, token))
    }

    @Override
    Result tokenValid(String token) {
        try {
            jwtUtil.parseJWT(token)
            return new Result(true, "验证成功")
        } catch (Exception e) {
            e.printStackTrace()
            return new Result(false, "自动登录失败，请重新登录")
        }
    }

    private redisRecordEmailToUser(User user) {
        def operations = redisTemplate.opsForValue()
        operations.set(RedisConstant.ACCOUNT_WAIT_CONFIRM + user.hashCode(), user, 10, TimeUnit.MINUTES)
        operations.set(RedisConstant.ACCOUNT_WAIT_CONFIRM_EMAIL + user.email, user, 10, TimeUnit.MINUTES)
        operations.set(RedisConstant.ACCOUNT_WAIT_CONFIRM_USERNAME + user.username, user, 10, TimeUnit.MINUTES)
    }

    private redisDeleteUserEmail(User user, String hash) {
        redisTemplate.delete(RedisConstant.ACCOUNT_WAIT_CONFIRM + hash)
        redisTemplate.delete(RedisConstant.ACCOUNT_WAIT_CONFIRM_EMAIL + user.email)
        redisTemplate.delete(RedisConstant.ACCOUNT_WAIT_CONFIRM_USERNAME + user.username)
    }

    @Autowired
    @Resource
    void setUserRepository(UserRepository userRepository) {
        this.userRepository = userRepository
    }

    @Autowired
    @Resource
    void setRedisTemplate(RedisTemplate<String, Object> redisTemplate) {
        this.redisTemplate = redisTemplate
    }

    @Autowired
    @Resource
    void setRabbitTemplate(RabbitTemplate rabbitTemplate) {
        this.rabbitTemplate = rabbitTemplate
    }

    @Autowired
    @Resource
    void setEncoder(BCryptPasswordEncoder encoder) {
        this.encoder = encoder
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }
}
